@@ -22,7 +22,6 @@ def getASG_DFARules( AssemblySystemName, component,model):
              :return objectList DFARuleList: list of interface object of the DFA rule (-1 = error)
     """
     # match the AssemblySystem-of-interest
-    DFARuleList = []
     if len(model.includesAssemblySystem.items) != 0:
         # match the assemblySystem by Name
         for assemblySystem in model.includesAssemblySystem.items:
@@ -105,7 +104,7 @@ def getDFArules( ruleContainer):
     """
       For internal use only!
     """
-    ruleList = []  # rulelist = [rule1,rule2,...,rulen] with rule = [name,description,isAppliedToProduct,isAppliedToProductPart,isAppliedToAssemblySequence,hasScorePropagation,hasScoreType,[DFArule.hasPreprocessesData.items[0].hasDataType,DFArule.hasPreprocessesData.items[0].hasDataReference]]
+    ruleList = []
     for DFArule in ruleContainer.hasDFARule.items:
         ruleDefinition = []
 
@@ -139,15 +138,15 @@ def getDFArules( ruleContainer):
         ruleDefinition.append(DFArule.isAppliedToAssemblySequence)
         ruleDefinition.append(DFArule.hasScorePropagation)
         ruleDefinition.append(DFArule.hasScoreType)
-        if DFArule.hasPreprocessedData is not None:  # TODO:check if with items or not
+        if DFArule.hasPreprocessedData is not None:
             ruleDefinition.append([DFArule.hasPreprocessesData.items[0].hasDataType,
                                    DFArule.hasPreprocessesData.items[0].hasDataReference])
         optionList = []
-        if len(DFArule.hasOption.items) != 0:  # TODO:check if with items or not
+        if len(DFArule.hasOption.items) != 0:
             for option in DFArule.hasOption.items:
                 optList = []
                 try:
-                    if option.hasValue is not None:  # TODO: check if we can do this like that!
+                    if option.hasValue is not None:
                         optList.append(option.hasValue)
                 except:
                     x = 10
@@ -172,7 +171,7 @@ def getDFArules( ruleContainer):
                 except:
                     x = 10
 
-                optionList.append([option.hasName, optList])  # TODO: check the other options how to check
+                optionList.append([option.hasName, optList])
         ruleDefinition.append(optionList)
 
         # check the subtyping
@@ -191,8 +190,9 @@ def getDFArules( ruleContainer):
                 except:
                     x = 10
 
-        except:  # AS-deptendent
-            x = 1  # TODO
+        except:  # AS-dependent
+            x = 1
+            # TODO: Add the AS-Dependent elements
 
         # transform to interface object for DFA rule
         rule_interface = DFARule(RuleType=ruleDefinition[0], Name=ruleDefinition[1],
@@ -200,7 +200,7 @@ def getDFArules( ruleContainer):
                                  isAppliedToProductPart=ruleDefinition[4],
                                  isAppliedToAssemblySequence=ruleDefinition[5],
                                  hasScorePropagation=ruleDefinition[6], hasScoreType=ruleDefinition[7])
-        if DFArule.hasPreprocessedData is not None:  # TODO:check if with items or not
+        if DFArule.hasPreprocessedData is not None:
             rule_interface.preprocessedData = ruleDefinition[8]
         rule_interface.optionList = optionList
         try:
@@ -229,8 +229,9 @@ def addASG_DFARule( AssemblySystemName, component,rule,model,KBPath,API):
     #
     #       :return int Error: -1 = error, 1= function performed correcly
     #     """
-    #     # match the AssemblySystem-of-interest
-    DFARuleList = []
+    #
+
+    # match the AssemblySystem-of-interest
     if len(model.includesAssemblySystem.items) != 0:
         # match the assemblySystem by Name
         for assemblySystem in model.includesAssemblySystem.items:
@@ -242,8 +243,6 @@ def addASG_DFARule( AssemblySystemName, component,rule,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleSelector
-                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
-                            model,cmp = DFA_Hack(API=API,AssemblySystemName=AssemblySystemName,classID="RuleSelector",cmp=cmp,model=model)
                             addDFArule(cmp,rule,API)
                             updateKBv6(KBPath,model)
                         except:
@@ -254,8 +253,6 @@ def addASG_DFARule( AssemblySystemName, component,rule,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleEvaluator
-                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
-                            model,cmp = DFA_Hack(API=API, AssemblySystemName=AssemblySystemName, classID="RuleEvaluator",cmp=cmp,model=model)
                             addDFArule(cmp, rule,API)
                             updateKBv6(KBPath,model)
                         except:
@@ -282,8 +279,10 @@ def addASG_DFARules( AssemblySystemName, component,ruleList,model,KBPath,API):
     #
     #       :return int Error: -1 = error, 1= function performed correcly
     #     """
-    #     # match the AssemblySystem-of-interest
-    DFARuleList = []
+    #
+
+    # match the AssemblySystem-of-interest
+
     if len(model.includesAssemblySystem.items) != 0:
         # match the assemblySystem by Name
         for assemblySystem in model.includesAssemblySystem.items:
@@ -295,8 +294,6 @@ def addASG_DFARules( AssemblySystemName, component,ruleList,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleSelector
-                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
-                            model, cmp = DFA_Hack(API=API, AssemblySystemName=AssemblySystemName,classID="RuleSelector", cmp=cmp, model=model)
                             for rule in ruleList:
                                 addDFArule(cmp,rule,API)
                             updateKBv6(KBPath,model)
@@ -308,12 +305,10 @@ def addASG_DFARules( AssemblySystemName, component,ruleList,model,KBPath,API):
                         #check if rule selector is available
                         try:
                             cmp = ASG_temp.hasRuleEvaluator
-                            # --!HACK!--:check if an DFARule class already exists, if not, generate a placeholder
-                            model, cmp = DFA_Hack(API=API, AssemblySystemName=AssemblySystemName,classID="RuleEvaluator", cmp=cmp, model=model)
                             for rule in ruleList:
                                 addDFArule(cmp, rule, API)
                             updateKBv6(KBPath,model)
-                        except:
+                        except Exception as e:
                             x = "No Rule evaluator available"
                             pass
 
@@ -333,20 +328,19 @@ def updateDFArule(self, ruleContainer, rule):
             DFArule.isAppliedToProduct = rule.isAppliedToProduct
             DFArule.isAppliedToProductPart = rule.isAppliedToProductPart
             DFArule.isAppliedToAssemblySequence = rule.isAppliedToAssemblySequence
-            # DFArule.hasScorePropagation  = rule[6]
-            # DFArule.hasScoreType  = rule[7]
+            # DFArule.hasScorePropagation  = rule[6]    #TODO: check how to add
+            # DFArule.hasScoreType  = rule[7]           #TODO: check how to add
             # update the preprocess data
-            if DFArule.hasPreprocessedData is not None:  # TODO:check if with items or not
+            if DFArule.hasPreprocessedData is not None:
                 DFArule.hasPreprocessesData.items[0].hasDataType = rule[8][0]
                 DFArule.hasPreprocessesData.items[0].hasDataReference = rule[8][1]
             # update the options
-            # TODO!!!
+            # TODO: Add the options - no example available to validate
 
 def addDFArule(ruleContainer, newRule,API):
     """
       For internal use only!
     """
-    API.initialize(API.ECORE_path)
     # generate a new ASG config and configure
     rule = API.create_noPlatformRoot("DFA_Rule")
     rule.hasName = newRule.Name
@@ -354,28 +348,16 @@ def addDFArule(ruleContainer, newRule,API):
     rule.isAppliedToProduct = newRule.isAppliedToProduct
     rule.isAppliedToProductPart = newRule.isAppliedToProductPart
     rule.isAppliedToAssemblySequence = newRule.isAppliedToAssemblySequence
-    # DFArule.hasScorePropagation  = rule[6]
-    # DFArule.hasScoreType  = rule[7]
+    # DFArule.hasScorePropagation  = rule[6]    #TODO:Chech how to add
+    # DFArule.hasScoreType  = rule[7]           #TODO:Check how to add
     # update the preprocess data
-    if rule.hasPreprocessedData is not None:  # TODO:check if with items or not
+    if rule.hasPreprocessedData is not None:
         rule.hasPreprocessesData.items[0].hasDataType = newRule[8][0]
         rule.hasPreprocessesData.items[0].hasDataReference = newRule[8][1]
         # update the options
-        # TODO!!!
+        # TODO: Add options - no example available to validate
 
-    ruleContainer.hasDFARule.items.append(rule)
-
-def DFA_Hack(API,AssemblySystemName,classID,cmp,model):
-    """
-      For internal use only!
-    """
-    if len(cmp.hasDFARule.items) == 0:
-        print("DOES NOT EXIST, PERFORM HACK")
-        model = EmptySubClassHack(API=API, classID=classID, className=cmp.hasName)
-        for assemblySystem in model.includesAssemblySystem.items:
-            if AssemblySystemName == assemblySystem.hasName:
-                ASG_temp = assemblySystem.hasAssemblyAlgorithmConfiguration
-                cmp = ASG_temp.hasRuleSelector
-        return model,cmp
-    else:
-        return model,cmp
\ No newline at end of file
+    try:
+        ruleContainer.hasDFARule.append(rule)
+    except Exception as e:
+        print(e)
