@@ -9,16 +9,14 @@ from pyecore.utils import DynamicEPackage
 #----------------------------------------------------------------------------------------------------------------------------------------------------
 def updateKB(model):
     """
-      For internal use only!
+      For internal use only! - OBSOLETE
     """
-    #TODO: update the current MM instead of the new one
     export_metamodel('C:/Users/B.MKR/Documents/06_PhD/03_Projects/03_PaCo/paco-workspace/PythonProjects/Master2/UA/API/unit-tests/output/OldGearBox.xmi',model)
 
 def updateKBv6(KB_path,model):
     """
       For internal use only!
     """
-    #TODO: update the current MM instead of the new one
     export_metamodel(KB_path,model)
     #necessary post-processing functions due to prefix error
     postProcess_prefix(KB_path)
@@ -28,13 +26,7 @@ def updateKBv6(KB_path,model):
     postProcess_ASG(KB_path)
     postProcess_StopCondition(KB_path)
     postProcess_platformRoot(KB_path)
-    #--DEBUG EXPERIMENT:use a blank template of optimizationProblem <includesOptimizationProblem/> and delete it
-    postProcess_EmptyOptimization(KB_path)
-    postProcess_EmptyProduct(KB_path)
-    postProcess_EmptyProductPart(KB_path)
-    postProcess_EmptyASG(KB_path)
-    #postProcess_EmptyStopCriterion(KB_path)
-    postProcess_EmptyDFARule(KB_path)
+    #--DEBUG EXPERIMENT:cleaning XMI manually
     postProcess_Line(KB_path)
     #--END DEBUG EXPERIMENT
 
@@ -44,7 +36,7 @@ def resolvePath(relativePath):
     """
     return os.path.abspath(relativePath)
 
-def importInstanceModel(path_ecore, path_instance, VERBOSE=False):
+def importInstanceModel(path_ecore, path_instance,init=True, VERBOSE=False):
     """
       For internal use only!
     """
@@ -65,6 +57,35 @@ def importInstanceModel(path_ecore, path_instance, VERBOSE=False):
 
     return model
 
+def importInstanceModel_NEW(path_ecore, path_instance,init=True, VERBOSE=False):
+    """
+      For internal use only!
+    """
+    rset = ResourceSet()
+    # add external mm resources
+    rset = addXMLResource(rset)
+
+    resource = rset.get_resource(URI(path_ecore))
+    mm_root = resource.contents[0]
+    rset.metamodel_registry[mm_root.nsURI] = mm_root
+    # At this point, the .ecore is loaded in the 'rset' as a metamodel
+    resource = rset.get_resource(URI(path_instance))
+    model_root = resource.contents[0]
+    model = model_root
+
+    # -- Internally instantiate MM to overcome double loading (SUGGESTION PYECORE DEV) --
+    res = rset.get_resource(path_ecore)
+    root_package = res.contents[0]
+    # Create a dynamic metamodel able to create instances of EClasses
+    MM = DynamicEPackage(root_package)
+    # Instantiate the root object -> this is hard coded for PACo
+    root_object = MM.PlatformRoot()
+    MM = MM
+    model_instance = root_object
+    #------------------------------------------------------------------------------------
+
+    return MM,model,model_instance
+
 def addXMLResource(rset):
     """
       For internal use only!
@@ -172,6 +193,30 @@ def create( e_class,MM,model_instance):
 
     return e_object
 
+def create_connected( e_class,MM,model_instance):
+    """
+        Generic create function to create instances of a specific EClass as defined by the MetaModel
+        and automatically connect it
+
+       :param e_class class: The EClass to create an instance of, specified either as a string or as an EClass
+       :return object createdObject: The created EObject
+    """
+
+    # Check if e_class contains a valid value
+    if isinstance(e_class, str):
+        e_class = getattr(MM, e_class)
+    if e_class not in list(vars(MM).values()):
+        raise TypeError(f"e_class of type {type(e_class)} not recognized.")
+
+    # Create the object
+    e_object = e_class()
+
+    # Attach the created object to the root object
+    root_reference = getattr(model_instance, 'includes' + e_class.name)
+    root_reference.append(e_object)            #original
+
+    return e_object
+
 def create_custom( object,e_class,MM):
     """
         Generic create function to create instances of a specific EClass as defined by the MetaModel
@@ -217,15 +262,8 @@ def create_noPlatformRoot( e_class,MM,model_instance):
     # Create the object
     e_object = e_class()
 
-    # Attach the created object to the root object
-    #root_reference = getattr(model_instance, 'includes' + e_class.name)
-    #root_reference.append(e_object)            #original
-    #root_reference.items.append(e_object)      #updated
-
     return e_object
 
-
-
 def export_metamodel( output_file,model):
     """
         Exports the python data into an xmi file defined by an Ecore model
@@ -369,127 +407,6 @@ def postProcess_ASG( output_file):
 
     file.close()
 
-#--DEBUG EXPERIMENT:use a blank template of optimizationProblem <includesOptimizationProblem/> and delete it
-def postProcess_EmptyOptimization( output_file):
-    """
-        Custom postprocessing function to clean the xmi file defined by an Ecore model
-
-        :param string path: path of the output xmi file
-    """
-    # open file to manually add missing attribute in root element (@ToDo clean this up)
-    with open(output_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '<includesOptimizationProblem/>'
-    replacement = ''
-
-    with open(output_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def postProcess_EmptyProduct( output_file):
-    """
-        Custom postprocessing function to clean the xmi file defined by an Ecore model
-
-        :param string path: path of the output xmi file
-    """
-    # open file to manually add missing attribute in root element (@ToDo clean this up)
-    with open(output_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '<includesProduct/>'
-    replacement = ''
-
-    with open(output_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def postProcess_EmptyProductPart( output_file):
-    """
-        Custom postprocessing function to clean the xmi file defined by an Ecore model
-
-        :param string path: path of the output xmi file
-    """
-    # open file to manually add missing attribute in root element (@ToDo clean this up)
-    with open(output_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '<includesProductPart/>'
-    replacement = ''
-
-    with open(output_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def postProcess_EmptyASG( output_file):
-    """
-        Custom postprocessing function to clean the xmi file defined by an Ecore model
-
-        :param string path: path of the output xmi file
-    """
-    # open file to manually add missing attribute in root element (@ToDo clean this up)
-    with open(output_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '<includesAssemblySystem/>'
-    replacement = ''
-
-    with open(output_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def postProcess_EmptyStopCriterion( output_file):
-    """
-        Custom postprocessing function to clean the xmi file defined by an Ecore model
-
-        :param string path: path of the output xmi file
-    """
-    # open file to manually add missing attribute in root element (@ToDo clean this up)
-    with open(output_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '<hasStopCriterion/>'
-    replacement = ''
-
-    with open(output_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def postProcess_EmptyDFARule( output_file):
-    """
-        Custom postprocessing function to clean the xmi file defined by an Ecore model
-
-        :param string path: path of the output xmi file
-    """
-    # open file to manually add missing attribute in root element (@ToDo clean this up)
-    with open(output_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '<hasDFARule/>'
-    replacement = ''
-
-    with open(output_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
 def postProcess_Line( output_file):
     """
         Custom postprocessing function to clean the xmi file defined by an Ecore model
@@ -502,85 +419,6 @@ def postProcess_Line( output_file):
                 writer.write(line)
         writer.truncate()
 
-
-def preProcess_emptyClass(KB_file,elementName="OptimizationProblem"):
-    """
-        Custom preprocessing function to generate an empty class element, used in the EmptyClassHack function
-
-        :param string KB_file: path of the KB instance
-        :param string elementName: name of the element to be added
-    """
-
-    with open(KB_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    target = '  <includesOperator/>'
-    replacement = '  <includes'+elementName+'/>\n  <includesOperator/>'
-
-    with open(KB_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def EmptyClassHack(API=None,elementName="OptimizationProblem"):
-    """
-        Custom hack function to generate an empty class element
-
-        :param string KB_file: path of the KB instance
-        :param string elementName: name of the element to be added
-
-        :return object model: KB instance model
-    """
-
-    preProcess_emptyClass(KB_file=API.KB_path, elementName=elementName)
-    API.model = importInstanceModel(API.ECORE_path, API.KB_path, VERBOSE=False)
-    return API.model
-
-#END DEBUG EXPERIMENT
-
-#--DEBUG EXPERIMENT: hacking the empty subclass e.g. to update stopconditions or DFA rules
-def preProcess_emptySubClass(KB_file,classID ="Terminator" ,className="Default"):
-    """
-        Custom preprocessing function to generate an empty class element, used in the EmptyClassHack function
-
-        :param string KB_file: path of the KB instance
-        :param string elementName: name of the element to be added
-    """
-
-    with open(KB_file, "r") as file:
-        data = file.read()
-
-    # Prefix replacement
-    if classID =="Terminator":
-        target = '      <has'+classID+' hasName="'+className+'"/>'
-        replacement = '      <has'+classID+' hasName="'+className+'">\n        <hasStopCriterion/>\n      </has'+classID+'>'
-    elif classID =="RuleSelector" or classID =="RuleEvaluator":
-        target = '      <has' + classID + ' hasName="' + className + '"/>'
-        replacement = '      <has' + classID + ' hasName="' + className + '">\n        <hasDFARule/>\n      </has' + classID + '>'
-
-    with open(KB_file, "w") as file:
-        data_new = data.replace(target, replacement)
-        file.write(data_new)
-
-    file.close()
-
-def EmptySubClassHack(API=None,classID ="Terminator" ,className="Default"):
-    """
-        Custom hack function to generate an empty subclass element
-
-        :param string KB_file: path of the KB instance
-        :param string elementName: name of the element to be added
-
-        :return object model: KB instance model
-    """
-
-    preProcess_emptySubClass(KB_file=API.KB_path,classID =classID ,className=className)
-    API.model = importInstanceModel(API.ECORE_path, API.KB_path, VERBOSE=False)
-    return API.model
-
-
 def postProcess_StopCondition( output_file):
     """
         Custom postprocessing function to clean the xmi file defined by an Ecore model
@@ -600,4 +438,212 @@ def postProcess_StopCondition( output_file):
         file.write(data_new)
 
     file.close()
-#END DEBUG EXPERIMENT
\ No newline at end of file
+
+#------------------------------------------------------------
+#           OBSOLETE HELPER FUNCTIONS
+#------------------------------------------------------------
+
+# -- DEBUG EXPERIMENT: empty eOrderedSet not updated - OBSOLETE
+
+# def postProcess_EmptyOptimization( output_file):
+#     """
+#         Custom postprocessing function to clean the xmi file defined by an Ecore model
+#
+#         :param string path: path of the output xmi file
+#     """
+#     # open file to manually add missing attribute in root element (@ToDo clean this up)
+#     with open(output_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '<includesOptimizationProblem/>'
+#     replacement = ''
+#
+#     with open(output_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def postProcess_EmptyProduct( output_file):
+#     """
+#         Custom postprocessing function to clean the xmi file defined by an Ecore model
+#
+#         :param string path: path of the output xmi file
+#     """
+#     # open file to manually add missing attribute in root element (@ToDo clean this up)
+#     with open(output_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '<includesProduct/>'
+#     replacement = ''
+#
+#     with open(output_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def postProcess_EmptyProductPart( output_file):
+#     """
+#         Custom postprocessing function to clean the xmi file defined by an Ecore model
+#
+#         :param string path: path of the output xmi file
+#     """
+#     # open file to manually add missing attribute in root element (@ToDo clean this up)
+#     with open(output_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '<includesProductPart/>'
+#     replacement = ''
+#
+#     with open(output_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def postProcess_EmptyASG( output_file):
+#     """
+#         Custom postprocessing function to clean the xmi file defined by an Ecore model
+#
+#         :param string path: path of the output xmi file
+#     """
+#     # open file to manually add missing attribute in root element (@ToDo clean this up)
+#     with open(output_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '<includesAssemblySystem/>'
+#     replacement = ''
+#
+#     with open(output_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def postProcess_EmptyStopCriterion( output_file):
+#     """
+#         Custom postprocessing function to clean the xmi file defined by an Ecore model
+#
+#         :param string path: path of the output xmi file
+#     """
+#     # open file to manually add missing attribute in root element (@ToDo clean this up)
+#     with open(output_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '<hasStopCriterion/>'
+#     replacement = ''
+#
+#     with open(output_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def postProcess_EmptyDFARule( output_file):
+#     """
+#         Custom postprocessing function to clean the xmi file defined by an Ecore model
+#
+#         :param string path: path of the output xmi file
+#     """
+#     # open file to manually add missing attribute in root element (@ToDo clean this up)
+#     with open(output_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '<hasDFARule/>'
+#     replacement = ''
+#
+#     with open(output_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def preProcess_emptyClass(KB_file,elementName="OptimizationProblem"):
+#     """
+#         Custom preprocessing function to generate an empty class element, used in the EmptyClassHack function
+#
+#         :param string KB_file: path of the KB instance
+#         :param string elementName: name of the element to be added
+#     """
+#
+#     with open(KB_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     target = '  <includesOperator/>'
+#     replacement = '  <includes'+elementName+'/>\n  <includesOperator/>'
+#
+#     with open(KB_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def EmptyClassHack(API=None,elementName="OptimizationProblem"):
+#     """
+#         Custom hack function to generate an empty class element
+#
+#         :param string KB_file: path of the KB instance
+#         :param string elementName: name of the element to be added
+#
+#         :return object model: KB instance model
+#     """
+#
+#     preProcess_emptyClass(KB_file=API.KB_path, elementName=elementName)
+#     API.model = importInstanceModel(API.ECORE_path, API.KB_path, VERBOSE=False)
+#     return API.model
+
+#END DEBUG EXPERIMENT
+
+# -- DEBUG EXPERIMENT: hacking the empty subclass - OBSOLETE
+
+# def preProcess_emptySubClass(KB_file,classID ="Terminator" ,className="Default"):
+#     """
+#         Custom preprocessing function to generate an empty class element, used in the EmptyClassHack function
+#
+#         :param string KB_file: path of the KB instance
+#         :param string elementName: name of the element to be added
+#     """
+#
+#     with open(KB_file, "r") as file:
+#         data = file.read()
+#
+#     # Prefix replacement
+#     if classID =="Terminator":
+#         target = '      <has'+classID+' hasName="'+className+'"/>'
+#         replacement = '      <has'+classID+' hasName="'+className+'">\n        <hasStopCriterion/>\n      </has'+classID+'>'
+#     elif classID =="RuleSelector" or classID =="RuleEvaluator":
+#         target = '      <has' + classID + ' hasName="' + className + '"/>'
+#         replacement = '      <has' + classID + ' hasName="' + className + '">\n        <hasDFARule/>\n      </has' + classID + '>'
+#
+#     with open(KB_file, "w") as file:
+#         data_new = data.replace(target, replacement)
+#         file.write(data_new)
+#
+#     file.close()
+
+# def EmptySubClassHack(API=None,classID ="Terminator" ,className="Default"):
+#     """
+#         Custom hack function to generate an empty subclass element
+#
+#         :param string KB_file: path of the KB instance
+#         :param string elementName: name of the element to be added
+#
+#         :return object model: KB instance model
+#     """
+#
+#     preProcess_emptySubClass(KB_file=API.KB_path,classID =classID ,className=className)
+#     API.model = importInstanceModel(API.ECORE_path, API.KB_path, VERBOSE=False)
+#     return API.model
+
+#END DEBUG EXPERIMENT
+
+
+
